import os
from dotenv import load_dotenv
import traceback

import supervisely as sly
from rich.console import Console


def remove_file_run(team_id:int, path:str) -> bool:

    if None in (os.environ.get('SERVER_ADDRESS'), os.environ.get('API_TOKEN')):
        load_dotenv(os.path.expanduser("~/supervisely.env"))

    console = Console()
    api = sly.Api.from_env()

    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} not exists\n", style='bold red')
        return False 

    try:
        api.file.remove_file(team_id, path)
        console.print(f"\nFile '{path}' successfully removed\n", style='bold green')
        return True
    
    except:
        console.print(f"\nRemoving file failed\n", style='bold red')
        traceback.print_exc()
        return False
    
def remove_dir_run(team_id:int, path:str) -> bool:

    console = Console()
    api = sly.Api.from_env()
    
    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} not exists\n", style='bold red')
        return False 
    
    try:
        api.file.remove_dir(team_id, path)
        console.print(f"\nDirectory '{path}' successfully removed\n", style='bold green')
        return True
    
    except:
        console.print(f"\nRemoving directory failed\n", style='bold red')
        traceback.print_exc()
        return False



import os
from dotenv import load_dotenv
import traceback

import supervisely as sly

def get_project_name_run(project_id:int) -> bool:

    if None in (os.environ.get('SERVER_ADDRESS'), os.environ.get('API_TOKEN')):
        load_dotenv(os.path.expanduser("~/supervisely.env"))

    api = sly.Api.from_env()

    try:
        project_info = api.project.get_info_by_id(project_id)
        if project_info is None:
            print(f"Project with id {project_id} is either archived, doesn't exist or you don't have enough permissions to access it")
            return False
        
        print(project_info.name)
        return True
    
    except:
        traceback.print_exc()
        return False
    




import os
import traceback
from dotenv import load_dotenv

from rich.console import Console
from tqdm import tqdm
import supervisely as sly


def download_run(id:int, dest_dir:str) -> bool:

    if None in (os.environ.get('SERVER_ADDRESS'), os.environ.get('API_TOKEN')):
        load_dotenv(os.path.expanduser("~/supervisely.env"))

    api = sly.Api.from_env()

    console = Console()
    console.print(f"\nDownloading data from project with ID={id} to directory: '{dest_dir}' ...\n", style="bold")

    project_info = api.project.get_info_by_id(id)

    if project_info is None:
        console.print('\nError: Project not exists\n', style="bold red")
        return False
            
    n_count = project_info.items_count
    try:
        if sly.is_development():
            with tqdm(total=n_count) as pbar:
                sly.download(api, id, dest_dir, progress_cb=pbar.update)
        else:
            sly.download(api, id, dest_dir, log_progress=True)
            
        return True
    except:
        console.print(f"\nProject is not downloaded\n", style='bold red')
        traceback.print_exc()
        return False


import sys
import click
from rich.console import Console


from supervisely.cli.download import download_run
from supervisely.cli.upload import upload_to_teamfiles_run, set_task_output_dir_run
from supervisely.cli.remove import remove_file_run, remove_dir_run
from supervisely.cli.get import get_project_name_run 


@click.group()
def cli():
    pass

@cli.command(
    help="This app allows you to release your aplication to Supervisely platform"
)
@click.option(
    "-p",
    "--path",
    required=False,
    help="[Optional] Path to the directory with application",
)
@click.option(
    "-a",
    "--sub-app",
    required=False,
    help="[Optional] Path to sub-app relative to application directory",
)
@click.option(
    "--release-version",
    required=False,
    help='[Optional] Release version in format "vX.X.X"',
)
@click.option("--release-description", required=False, help="[Optional] Release description (max length is 64 symbols)")
@click.option("-y", is_flag=True, help="[Optional] Add this flag for autoconfirm")
@click.option("-s", "--slug", required=False, help="[Optional] For internal use")
def release(path, sub_app, slug, y, release_version, release_description):
    from supervisely.cli.release import run
    try:
        success = run(
            app_directory=path,
            sub_app_directory=sub_app,
            slug=slug,
            autoconfirm=y,
            release_version=release_version,
            release_description=release_description,
        )
        if success:
            print("App released sucessfully!")
            sys.exit(0)
        else:
            print("App not released")
            sys.exit(1)
    except KeyboardInterrupt:
        print("Aborting...")
        print("App not released")
        sys.exit(1)


@cli.group()
def project():
    """Commands: download, get-name"""
    pass

@project.command(
    help="Download project data from supervisely to local directory"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely project ID",
)
@click.option(
    "-d",
    "--dst",
    required=True,
    type=str,
    help="Download destination directory",
)
def download(id: int, dst: str) -> None:
    console = Console()
    try:
        success = download_run(id, dst)
        if success:
            console.print("\nProject is downloaded sucessfully!\n", style="bold green")
            sys.exit(0)
        else:
            sys.exit(1)
    except KeyboardInterrupt:
        print("\nDownload aborted\n")
        sys.exit(1)

@project.command(
    help="Get project name"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely project ID",
)
def get_name(id:int) -> None:    
    try:
        success = get_project_name_run(id)
        if success:
            sys.exit(0)
        else:
            print("Getting project name failed")
            sys.exit(1)
    except KeyboardInterrupt:
        print("Getting project name directory aborted")
        sys.exit(1)


@cli.group()
def teamfiles():
    """Commands: remove-file, remove-dir, upload"""
    pass

@teamfiles.command(
    help="Remove file from supervisely teamfiles"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely team ID",
)
@click.option(
    "-p",
    "--path",
    required=True,
    type=str,
    help="File path to remove",
)
def remove_file(id:int, path:str) -> None:
    try:
        success = remove_file_run(id, path)
        if success:
            sys.exit(0)
        else:
            sys.exit(1)
    except KeyboardInterrupt:
        print("\nRemoving file aborted\n")
        sys.exit(1)

@teamfiles.command(
    help="Remove directory from supervisely teamfiles"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely team ID",
)
@click.option(
    "-p",
    "--path",
    required=True,
    type=str,
    help="Path to remove directory",
)
def remove_dir(id:int, path:str) -> None:
    try:
        success = remove_dir_run(id, path)
        if success:
            sys.exit(0)
        else:
            sys.exit(1)
    except KeyboardInterrupt:
        print("Removing directory aborted")
        sys.exit(1)

@teamfiles.command(
    help="Upload local source files with destination to supervisely teamfiles"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely team ID",
)
@click.option(
    "-s",
    "--src",
    required=True,
    type=str,
    help="Path to local source directory from which files are uploaded",
)
@click.option(
    "-d",
    "--dst",
    required=True,
    type=str,
    help="Path to teamfiles remote destination directory to which files are uploaded",
)
def upload(id:int, src:str, dst:str) -> None:
    console = Console()
    try:
        success = upload_to_teamfiles_run(id, src, dst)
        if success:
            console.print("\nLocal directory uploaded to teamfiles sucessfully!\n", style='bold green')
            sys.exit(0)
        else:
            sys.exit(1)
    except KeyboardInterrupt:
        print("Upload aborted")
        sys.exit(1)

@cli.group()
def task():
    """Commands: set-output-dir"""
    pass

@task.command(
    help="Set link to teamfiles directory at workspace tasks interface"
)
@click.option(
    "-id",
    "--id",
    required=True,
    type=int,
    help="Supervisely task ID",
)
@click.option(
    "--team-id",
    required=True,
    type=int,
    help="Supervisely team ID",
)
@click.option(
    "-d",
    "--dir",
    required=True,
    type=str,
    help="Path to teamfiles directory",
)
def set_output_dir(id:int, team_id:int, dir:str) -> None:
    try:
        success = set_task_output_dir_run(id, team_id, dir)
        if success:
            sys.exit(0)
        else:
            sys.exit(1)
    except KeyboardInterrupt:
        print("Setting task output directory aborted")
        sys.exit(1)



import os
import supervisely as sly
from functools import partial
from dotenv import load_dotenv

import json
from tqdm import tqdm

import traceback
from rich.console import Console


def upload_to_teamfiles_run(team_id:int, local_dir:str, remote_dir:str) -> bool:

    if None in (os.environ.get('SERVER_ADDRESS'), os.environ.get('API_TOKEN')):
        load_dotenv(os.path.expanduser("~/supervisely.env"))

    console = Console()
    api = sly.Api.from_env()

    progress_bar = sly.app.widgets.Progress()

    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} not exists\n", style='bold red')
        return False         
    if not (os.path.exists(local_dir) and os.path.isdir(local_dir) ):
        console.print(f"\nError: local directory '{local_dir}' not exists\n", style='bold red')
        return False

    class ProgressBar(tqdm):
        def update_to(self, n: int) -> None:
            self.update(n - self.n)

    def upload_monitor_console(monitor, progress: sly.Progress, tqdm_pb: ProgressBar):
        if progress.total == 0:
            progress.set(monitor.bytes_read, monitor.len, report=False)
            tqdm_pb.total = monitor.len
        else:
            progress.set_current_value(monitor.bytes_read, report=False)
            tqdm_pb.update_to(monitor.bytes_read)

    def upload_monitor_instance(monitor, progress):
        readed_percent = round(monitor.bytes_read / monitor.len * 100)
        progress.update(readed_percent - progress.n)


    console.print(f"\nUploading local directory from '{local_dir}' to teamfiles directory: '{remote_dir}' ...\n", style="bold")

    try:
    
        if sly.is_development():
            progress = sly.Progress("Uploading training results directory to teamfiles...", 0, is_size=True)
            progress_size_cb = partial(upload_monitor_console, progress=progress, tqdm_pb=ProgressBar())
            api.file.upload_directory(team_id, local_dir, remote_dir, change_name_if_conflict=True, progress_size_cb=progress_size_cb)
        
        else:
            with progress_bar(message=f"Uploading training results directory to teamfiles...", total=100) as pbar:
                progress_size_cb = partial(upload_monitor_instance, progress=pbar)
                api.file.upload_directory(team_id, local_dir, remote_dir, change_name_if_conflict=True, progress_size_cb=progress_size_cb)
                pbar.message = 'Success'
                pbar.refresh()
                        
        return True
    
    except:
        console.print("\nUpload failed\n", style='bold red')
        traceback.print_exc()
        return False

def set_task_output_dir_run(task_id:int, team_id:int, dst_dir:str) -> bool:
    
    console = Console()
    api = sly.Api.from_env()

    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} not exists\n", style='bold red')
        return False     
    if api.task.get_info_by_id(task_id) is None:
        console.print(f"\nError: Task with ID={task_id} not exists\n", style='bold red')
        return False           
    if not api.file.dir_exists(team_id, dst_dir):
        console.print(f"\nError: directory '{dst_dir}' not exists in teamfiles\n", style='bold red')
        return False

       
    try:
        files = api.file.list2(team_id, dst_dir, recursive=True)

        if len(files) == 0:
            # some data to create dummy .json file to get file id
            data = {"team_id": team_id, "task_id": task_id, "directory": dst_dir}
            
            src_path = os.path.join( os.getcwd(), "info.json")
            with open(src_path, "w") as f:
                json.dump(data, f)
            
            dst_path = os.path.join( dst_dir, "info.json")
            file_id = api.file.upload( team_id, src_path, dst_path ).id

        else:
            file_id = files[0].id

        api.task.set_output_directory(task_id, file_id, dst_dir)
        console.print("\nSetting task output directory succeed\n", style='bold green')
        return True

    except:
        console.print("\nSetting task output directory failed\n", style='bold red')
        traceback.print_exc()
        return False
    
